-- --------------------------------------
-- NeedToKnow
-- by Nephthys <Drunken Monkeys> of Hyjal
-- zomg rawr
-- Inspired by DiscoDice
-- --------------------------------------


-- -------------
-- ADDON GLOBALS
-- -------------

NeedToKnow_Version = "1.0";

local NeedToKnow_BarUpdateInterval = 0.05;
local NeedToKnow_ButtonUpdateInterval = 0.5;

local NeedToKnow_DefaultFrameWidth = 200;
local NeedToKnow_DefaultBarWidth = 195;
local NeedToKnow_DefaultBarBorderWidth = 256;

local chatW = "|cffffffff";
local chatY = "|cffffff77";

--	DEFAULT_CHAT_FRAME:AddMessage("NeedToKnow test.");



-- ----------------
-- DEFAULT SETTINGS
-- ----------------

local NeedToKnow_Defaults = {
	["BarColor"] = 	{	{ r = 0.8, g = 0.6, b = 0.0 },
						{ r = 0.8, g = 0.2, b = 0.1 },
						{ r = 0.2, g = 0.7, b = 0.1 },
						{ r = 0.2, g = 0.1, b = 0.7 },
					}, 
	["BuffWatched"] = {"Slice and Dice", "Blade Flurry", "Find Weakness", "Ghostly Strike"}, 
	["CooldownsOn"] = true, 
	["CooldownsOnlyInCombat"] = false, 
	["CooldownWatched"] = { "Feint", "Cold Blood", "Adrenaline Rush", "Blade Flurry", "Preparation", "Ghostly Strike", "Cloak of Shadows" }, 
	["Locked"] = false,
	["Scale"] = 1.0,
	["Version"] = NeedToKnow_Version,
	["Width"] = 1.0,
};



-- ----------
-- MAIN FRAME
-- ----------

function NeedToKnow_Frame_OnLoad()
	this:RegisterEvent("PLAYER_LOGIN");
	SlashCmdList["NEEDTOKNOW"] = NeedToKnow_SlashCommand;
	SLASH_NEEDTOKNOW1 = "/needtoknow";
	SLASH_NEEDTOKNOW2 = "/ntk";
	DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow v"..NeedToKnow_Version..chatW.." loaded.  Type " ..chatY.."/ntk" ..chatW.." for help." );
end

function NeedToKnow_Frame_OnEvent(event)
	if ( event == "PLAYER_LOGIN" ) then				-- setup needs to be late enough that the icons can find their textures
		NeedToKnow_Settings = NeedToKnow_Settings or NeedToKnow_Defaults;
		if ( NeedToKnow_Settings["Version"] < NeedToKnow_Version ) then
			NeedToKnow_Settings_tmp = NeedToKnow_Defaults;
			for key,value in pairs(NeedToKnow_Settings) do
				if ( NeedToKnow_Defaults[key] ) then
					NeedToKnow_Settings_tmp[key] = value;
				end
			end
			NeedToKnow_Settings_tmp["Version"] = NeedToKnow_Version;
			NeedToKnow_Settings = NeedToKnow_Settings_tmp;
		end
		-- thanks to WoWWiki for the settings-transfer code
		NeedToKnow_SetAppearance();
	end
end

function NeedToKnow_SetAppearance()					-- called by OnEvent, slash commands
	local mainFrame = getglobal("NeedToKnow");
	local bottomText = getglobal("NeedToKnowBottomText");
	mainFrame:SetScale( NeedToKnow_Settings["Scale"] );
	mainFrame:SetWidth( NeedToKnow_DefaultFrameWidth * NeedToKnow_Settings["Width"] );
	if ( NeedToKnow_Settings["Locked"] ) then
		mainFrame:EnableMouse(0);
		bottomText:Hide();
	else
		mainFrame:EnableMouse(1);
		bottomText:SetText( chatY.."type '/ntk lock' when ready" );
		bottomText:Show();
	end
	for barNumber = 1,4 do 
		NeedToKnow_Bar_Initialize(barNumber);
	end 
	NeedToKnow_ButtonFrame_Initialize();
	for buttonNumber = 1,7 do 
		NeedToKnow_Button_Initialize(buttonNumber);
	end
end



-- --------------------
-- PLAYER CUSTOMIZATION
-- --------------------

function NeedToKnow_SlashCommand(cmd)

	if ( cmd == "on" ) then

	elseif ( cmd == "lock" ) then
		NeedToKnow_Settings["Locked"] = true;
		NeedToKnow_SetAppearance();
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." locked." );
	elseif ( cmd == "unlock" ) then
		NeedToKnow_Settings["Locked"] = false;
		NeedToKnow_SetAppearance();
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." unlocked." );

	elseif ( string.find(cmd, "scale") ) then
		local scale = 1 * string.gsub(cmd, "scale (.)", "%1");
		NeedToKnow_Settings["Scale"] = scale;
		NeedToKnow_SetAppearance();
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." scale set to "..scale );

	elseif ( string.find(cmd, "width") ) then
		local width = 1 * string.gsub(cmd, "width (.)", "%1");
		NeedToKnow_Settings["Width"] = width;
		NeedToKnow_SetAppearance();
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." width set to "..width );

	elseif ( string.find(cmd, "bar") ) then
		local barNumber = 1 * string.gsub(cmd, "bar (%d) (.*)", "%1");
		local buffWatched = string.gsub(cmd, "bar (%d) (.*)", "%2");
--		if ( buffWatched == "on" ) then
--			NeedToKnow_Settings["BarIsOn"][barNumber] = true;
--			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." Bar "..barNumber.." on." );
		if ( buffWatched == "off" or buffWatched == "none" ) then
			NeedToKnow_Settings["BuffWatched"][barNumber] = nil;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." Bar "..barNumber.." turned off." );
		elseif ( buffWatched ) then
			NeedToKnow_Settings["BuffWatched"][barNumber] = buffWatched;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." Bar "..barNumber.." watching "..buffWatched.."." );
		end
		NeedToKnow_Bar_Initialize(barNumber);

	elseif ( string.find(cmd, "icon") ) then
		local buttonNumber = 1 * string.gsub(cmd, "icon (%d) (.*)", "%1");
		local cooldownWatched = string.gsub(cmd, "icon (%d) (.*)", "%2");
		if ( cooldownWatched == "off" or cooldownWatched == "none" ) then
			NeedToKnow_Settings["CooldownWatched"][buttonNumber] = nil;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." Icon "..buttonNumber.." turned off." );
		elseif ( cooldownWatched ) then
			NeedToKnow_Settings["CooldownWatched"][buttonNumber] = cooldownWatched;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." Icon "..buttonNumber.." watching "..cooldownWatched.."." );
		end
		NeedToKnow_Button_Initialize(buttonNumber);

	elseif ( cmd == "onlyincombat" ) then
		if ( NeedToKnow_Settings["CooldownsOnlyInCombat"] ) then
			NeedToKnow_Settings["CooldownsOnlyInCombat"] = false;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." cooldown icons always shown." );
		elseif ( not NeedToKnow_Settings["CooldownsOnlyInCombat"] ) then
			NeedToKnow_Settings["CooldownsOnlyInCombat"] = true;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." cooldown icons only showing in combat." );
		end
		NeedToKnow_ButtonFrame_Initialize();

	elseif ( cmd == "reset" ) then
		NeedToKnow_Reset();
		NeedToKnow_SetAppearance();
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." reset to defaults.");

	elseif ( cmd == "cooldown" ) then
		if ( NeedToKnow_Settings["CooldownsOn"] ) then
			NeedToKnow_Settings["CooldownsOn"] = false;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." cooldown icons off." );
		elseif ( not NeedToKnow_Settings["CooldownsOn"] ) then
			NeedToKnow_Settings["CooldownsOn"] = true;
			DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow:"..chatW.." cooldown icons on." );
		end
		NeedToKnow_SetAppearance();

	else 
		DEFAULT_CHAT_FRAME:AddMessage( chatY.."NeedToKnow commands: " );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk " ..chatW.."and " ..chatY.."/needtoknow " ..chatW.."are interchangeable " );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk bar [number] [buff / off] " ..chatW.."- Set buff watched by bar, or turn it off.  Be careful of your spelling and capitalization." );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk icon [number] [ability / off] " ..chatW.."- Set cooldown watched by icon, or turn it off.  Be careful of your spelling and capitalization." );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk cooldown " ..chatW.."-  Toggle cooldown icons on/off" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk onlyincombat " ..chatW.."- Show cooldown icons only while in combat (toggle on/off)" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk lock " ..chatW.."- Hides display and makes it unmovable" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk unlock " ..chatW.."- Shows display and makes it movable" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk scale [number] " ..chatW.."- Set scale of display (1 is normal size)" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk width [number] " ..chatW.."- Set width of timer bars (1 is normal size)" );
		DEFAULT_CHAT_FRAME:AddMessage( "- " ..chatY.."/ntk reset " ..chatW.."- Reset to default settings" );

--		DEFAULT_CHAT_FRAME:AddMessage( "- /ntk [on/off] - Turn NeedToKnow on/off" );
--		DEFAULT_CHAT_FRAME:AddMessage( "- /ntk settings - Show current settings" );

	end

end

function NeedToKnow_Reset()
	NeedToKnow_Settings = {};
	NeedToKnow_CopyTable(NeedToKnow_Defaults, NeedToKnow_Settings);
	NeedToKnow:ClearAllPoints();
	NeedToKnow:SetPoint("CENTER", "UIParent", "CENTER", 0, 0);
end

function NeedToKnow_CopyTable(source, destination)
	for i, value in pairs(source) do
		if (type(value) == "table") then
			destination[i] = {};
			NeedToKnow_CopyTable(value, destination[i]);
		else
			destination[i] = value;
		end
	end
end
-- Thanks to eCastingBar for the CopyTable code



-- ----------
-- TIMER BARS
-- ----------

function NeedToKnow_Bar_Initialize(barNumber)					-- called by main frame and slash commands
	local thisBar = getglobal("NeedToKnow_Bar"..barNumber);
	local color = NeedToKnow_Settings["BarColor"][barNumber];
	local border = getglobal("NeedToKnow_Bar"..barNumber.."Border");
	local barSpark = getglobal("NeedToKnow_Bar"..barNumber.."Spark");
	local barText = getglobal("NeedToKnow_Bar"..barNumber.."Text");
	thisBar.TimeSinceLastUpdate = 0;
	thisBar.startTime = 0;
	thisBar.endTime = 0;
	thisBar:SetWidth( NeedToKnow_DefaultBarWidth * NeedToKnow_Settings["Width"] );
	thisBar:SetMinMaxValues(0, 100);
	thisBar:SetValue(100);
	thisBar:SetStatusBarColor(color.r, color.g, color.b);
	border:SetWidth( NeedToKnow_DefaultBarBorderWidth * NeedToKnow_Settings["Width"] );
	barSpark:SetPoint("CENTER", thisBar, "RIGHT");
	barText:SetText( NeedToKnow_Settings["BuffWatched"][barNumber] or "(off)" );
	thisBar:Show();
end

function NeedToKnow_Bar_OnLoad()
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
end

function NeedToKnow_Bar_OnEvent(event, arg1)
	if ( event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS" ) then
		local buffWatched = NeedToKnow_Settings["BuffWatched"][this:GetID()];
 		if ( buffWatched and string.gsub(arg1, "You gain (.*)\.", "%1") == buffWatched ) then
			this:Show();
		end
	end
end

function NeedToKnow_Bar_OnUpdate(self, elapsed)
	if ( NeedToKnow_Settings["Locked"] ) then 
		local buffWatched = NeedToKnow_Settings["BuffWatched"][this:GetID()];
		if ( buffWatched ) then
			this.TimeSinceLastUpdate = this.TimeSinceLastUpdate + elapsed;
			if ( this.TimeSinceLastUpdate > NeedToKnow_BarUpdateInterval ) then 
				local buffIsActive = false;
				for buffID = 1,16 do 
					if ( GetPlayerBuffName(buffID) == buffWatched ) then

						buffIsActive = true;

						local currentTime = GetTime();
						local timeLeft = GetPlayerBuffTimeLeft(buffID);
						if ( currentTime + timeLeft > this.endTime ) then
							this.startTime = currentTime;
							this.endTime = currentTime + timeLeft;
							this:SetMinMaxValues(0, timeLeft);
						elseif ( currentTime > this.endTime ) then
							currentTime = this.endTime;
						end
						this:SetValue(this.endTime - currentTime);

						local barText = getglobal(this:GetName().."Text");
						barText:SetText( SecondsToTimeAbbrev(timeLeft).." - "..buffWatched );

						local barSpark = getglobal(this:GetName().."Spark");
						local sparkPosition = ((this.endTime - currentTime) / (this.endTime - this.startTime)) * this:GetWidth();
						if ( sparkPosition < 0 ) then
							sparkPosition = 0;
						end
						barSpark:SetPoint("CENTER", this, "LEFT", sparkPosition, 0);

					end
					if buffIsActive then break end
				end
				if ( not buffIsActive ) then
					this:Hide();
				end
				this.TimeSinceLastUpdate = 0;
			end
		else 
			this:Hide();
		end
	end
end



-- --------------
-- COOLDOWN ICONS
-- --------------

function NeedToKnow_ButtonFrame_Initialize()					-- called by main frame and slash commands
	local buttonFrame = getglobal("NeedToKnow_ButtonFrame");
	if ( NeedToKnow_Settings["Locked"] and ( not NeedToKnow_Settings["CooldownsOn"] or NeedToKnow_Settings["CooldownsOnlyInCombat"] ) ) then
		buttonFrame:Hide();
	else
		buttonFrame:Show();
	end
end

function NeedToKnow_ButtonFrame_OnLoad()
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
end

function NeedToKnow_ButtonFrame_OnEvent(event)
	if ( NeedToKnow_Settings["Locked"] and NeedToKnow_Settings["CooldownsOn"] and NeedToKnow_Settings["CooldownsOnlyInCombat"] ) then
		if ( event == "PLAYER_REGEN_DISABLED" ) then
			this:Show();
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then
			this:Hide();
		end
	end
end

function NeedToKnow_Button_Initialize(buttonNumber)				-- called by main frame and slash commands
	local button = getglobal("NeedToKnow_ButtonFrame_Button"..buttonNumber);
	local icon = getglobal(button:GetName().."_Icon");
	local cooldownWatched = NeedToKnow_Settings["CooldownWatched"][buttonNumber];
	if ( cooldownWatched ) then 
		if ( GetSpellCooldown(cooldownWatched) ) then
			icon:SetTexture(GetSpellTexture(cooldownWatched));
		elseif ( GetItemCooldown(cooldownWatched) ) then
 			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(cooldownWatched);
			icon:SetTexture(itemTexture);
		end
	else
		icon:SetTexture(nil);
	end
	if ( NeedToKnow_Settings["Locked"] and not icon:GetTexture() ) then
		button:Hide();
	else 
		button:Show();
	end
	if ( not NeedToKnow_Settings["Locked"] and not icon:GetTexture() ) then
		icon:SetTexture(0, 0, 0, 0.5);
	end
	icon:Show();
	button.TimeSinceLastUpdate = 0;
end

function NeedToKnow_Button_OnUpdate(self, elapsed)
	if ( NeedToKnow_Settings["Locked"] ) then 
		this.TimeSinceLastUpdate = this.TimeSinceLastUpdate + elapsed;
		if ( this.TimeSinceLastUpdate > NeedToKnow_ButtonUpdateInterval ) then 
			local cooldownWatched = NeedToKnow_Settings["CooldownWatched"][this:GetID()];
			local startTime, timeLeft = GetSpellCooldown(cooldownWatched);
			local startTimeTmp, timeLeftTmp =  GetItemCooldown(cooldownWatched);
			timeLeft = timeLeft or timeLeftTmp;
			local icon = getglobal(this:GetName().."_Icon");
			local isVisible = icon:IsVisible();
			if ( timeLeft > 1.5 and isVisible ) then
				icon:Hide(); 
			elseif ( timeLeft == 0 and not isVisible ) then 
				icon:Show();
			end
			this.TimeSinceLastUpdate = 0;
		end
	end
end


